@@ -1,4 +1,4 @@
-# Copyright 2017-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,6 +10,7 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Placeholder docstring"""
 from __future__ import absolute_import
 import json
 
@@ -17,10 +18,9 @@ from sagemaker.utils import to_str
 
 
 class ParameterRange(object):
-    """Base class for representing parameter ranges. This is used to define
-    what hyperparameters to tune for an Amazon SageMaker hyperparameter tuning job and to verify
-    hyperparameters for Marketplace Algorithms.
-
+    """Base class for representing parameter ranges. This is used to define what
+    hyperparameters to tune for an Amazon SageMaker hyperparameter tuning job
+    and to verify hyperparameters for Marketplace Algorithms.
     """
 
     __all_types__ = ("Continuous", "Categorical", "Integer")
@@ -31,8 +31,9 @@ class ParameterRange(object):
         Args:
             min_value (float or int): The minimum value for the range.
             max_value (float or int): The maximum value for the range.
-            scaling_type (str): The scale used for searching the range during tuning (default: 'Auto').
-                Valid values: 'Auto', 'Linear', 'Logarithmic' and 'ReverseLogarithmic'.
+            scaling_type (str): The scale used for searching the range during
+                tuning (default: 'Auto'). Valid values: 'Auto', 'Linear',
+                'Logarithmic' and 'ReverseLogarithmic'.
         """
         self.min_value = min_value
         self.max_value = max_value
@@ -47,21 +48,26 @@ class ParameterRange(object):
         Returns:
             bool: True if valid, False otherwise.
         """
-        return value >= self.min_value and value <= self.max_value
+        return self.min_value <= value <= self.max_value
 
     @classmethod
     def cast_to_type(cls, value):
+        """
+        Args:
+            value:
+        """
         return float(value)
 
     def as_tuning_range(self, name):
-        """Represent the parameter range as a dicionary suitable for a request to
-        create an Amazon SageMaker hyperparameter tuning job.
+        """Represent the parameter range as a dicionary suitable for a request
+        to create an Amazon SageMaker hyperparameter tuning job.
 
         Args:
             name (str): The name of the hyperparameter.
 
         Returns:
-            dict[str, str]: A dictionary that contains the name and values of the hyperparameter.
+            dict[str, str]: A dictionary that contains the name and values of
+            the hyperparameter.
         """
         return {
             "Name": name,
@@ -73,6 +79,7 @@ class ParameterRange(object):
 
 class ContinuousParameter(ParameterRange):
     """A class for representing hyperparameters that have a continuous range of possible values.
+
     Args:
             min_value (float): The minimum value for the range.
             max_value (float): The maximum value for the range.
@@ -82,11 +89,16 @@ class ContinuousParameter(ParameterRange):
 
     @classmethod
     def cast_to_type(cls, value):
+        """
+        Args:
+            value:
+        """
         return float(value)
 
 
 class CategoricalParameter(ParameterRange):
-    """A class for representing hyperparameters that have a discrete list of possible values.
+    """A class for representing hyperparameters that have a discrete list of
+    possible values.
     """
 
     __name__ = "Categorical"
@@ -95,8 +107,8 @@ class CategoricalParameter(ParameterRange):
         """Initialize a ``CategoricalParameter``.
 
         Args:
-            values (list or object): The possible values for the hyperparameter. This input will
-                be converted into a list of strings.
+            values (list or object): The possible values for the hyperparameter.
+                This input will be converted into a list of strings.
         """
         if isinstance(values, list):
             self.values = [to_str(v) for v in values]
@@ -104,37 +116,48 @@ class CategoricalParameter(ParameterRange):
             self.values = [to_str(values)]
 
     def as_tuning_range(self, name):
-        """Represent the parameter range as a dicionary suitable for a request to
-        create an Amazon SageMaker hyperparameter tuning job.
+        """Represent the parameter range as a dicionary suitable for a request
+        to create an Amazon SageMaker hyperparameter tuning job.
 
         Args:
             name (str): The name of the hyperparameter.
 
         Returns:
-            dict[str, list[str]]: A dictionary that contains the name and values of the hyperparameter.
+            dict[str, list[str]]: A dictionary that contains the name and values
+            of the hyperparameter.
         """
         return {"Name": name, "Values": self.values}
 
     def as_json_range(self, name):
-        """Represent the parameter range as a dictionary suitable for a request to
-        create an Amazon SageMaker hyperparameter tuning job using one of the deep learning frameworks.
+        """Represent the parameter range as a dictionary suitable for a request
+        to create an Amazon SageMaker hyperparameter tuning job using one of the
+        deep learning frameworks.
 
-        The deep learning framework images require that hyperparameters be serialized as JSON.
+        The deep learning framework images require that hyperparameters be
+        serialized as JSON.
 
         Args:
             name (str): The name of the hyperparameter.
 
         Returns:
-            dict[str, list[str]]: A dictionary that contains the name and values of the hyperparameter,
-                where the values are serialized as JSON.
+            dict[str, list[str]]: A dictionary that contains the name and values of the
+            hyperparameter, where the values are serialized as JSON.
         """
         return {"Name": name, "Values": [json.dumps(v) for v in self.values]}
 
     def is_valid(self, value):
+        """
+        Args:
+            value:
+        """
         return value in self.values
 
     @classmethod
     def cast_to_type(cls, value):
+        """
+        Args:
+            value:
+        """
         return to_str(value)
 
 
@@ -149,4 +172,8 @@ class IntegerParameter(ParameterRange):
 
     @classmethod
     def cast_to_type(cls, value):
+        """
+        Args:
+            value:
+        """
         return int(value)
