@@ -1,4 +1,4 @@
-# Copyright 2017-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,6 +10,7 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Placeholder docstring"""
 from __future__ import absolute_import
 
 import contextlib
@@ -21,11 +22,13 @@ import tempfile
 import threading
 import time
 
+from sagemaker.debugger import DebuggerHookConfig
 from sagemaker.estimator import Framework
 import sagemaker.fw_utils as fw
-from sagemaker.tensorflow.defaults import TF_VERSION
+from sagemaker.tensorflow import defaults
 from sagemaker.tensorflow.model import TensorFlowModel
 from sagemaker.tensorflow.serving import Model
+from sagemaker.transformer import Transformer
 from sagemaker import utils
 from sagemaker.vpc_utils import VPC_CONFIG_DEFAULT
 
@@ -46,18 +49,21 @@ _SCRIPT_MODE_SERVING_ERROR_MSG = (
 )
 _SCRIPT_MODE_TENSORBOARD_WARNING = (
     "Tensorboard is not supported with script mode. You can run the following "
-    "command: tensorboard --logdir {} --host localhost --port 6006 This can be "
+    "command: tensorboard --logdir %s --host localhost --port 6006 This can be "
     "run from anywhere with access to the S3 URI used as the logdir."
 )
 
 
 class Tensorboard(threading.Thread):
+    """Placeholder docstring"""
+
     def __init__(self, estimator, logdir=None):
         """Initialize ``Tensorboard`` instance.
 
         Args:
             estimator (sagemaker.estimator.Framework): A SageMaker ``Estimator``.
-            logdir (str): Directory for logs (default: None). If not specified, a temporary directory is made.
+            logdir (str): Directory for logs (default: None). If not specified, a temporary
+                directory is made.
         """
         threading.Thread.__init__(self)
         self.event = threading.Event()
@@ -66,6 +72,7 @@ class Tensorboard(threading.Thread):
 
     @staticmethod
     def _cmd_exists(cmd):
+        """Placeholder docstring"""
         return any(
             os.access(os.path.join(path, cmd), os.X_OK)
             for path in os.environ["PATH"].split(os.pathsep)
@@ -173,7 +180,7 @@ class Tensorboard(threading.Thread):
         """Run TensorBoard process."""
         port, tensorboard_process = self.create_tensorboard_process()
 
-        logger.info("TensorBoard 0.1.7 at http://localhost:{}".format(port))
+        logger.info("TensorBoard 0.1.7 at http://localhost:%s", port)
         while not self.estimator.checkpoint_path:
             self.event.wait(1)
         with self._temporary_directory() as aws_sync_dir:
@@ -190,17 +197,21 @@ class TensorFlow(Framework):
 
     __framework_name__ = "tensorflow"
 
-    LATEST_VERSION = "1.13"
-    """The latest version of TensorFlow included in the SageMaker pre-built Docker images."""
+    LATEST_VERSION = defaults.LATEST_VERSION
+
+    _LATEST_1X_VERSION = "1.15.2"
 
     _LOWEST_SCRIPT_MODE_ONLY_VERSION = [1, 13]
+    # 2.0.0 still supports py2
+    # we will need to update this version number if future versions still support py2
+    _HIGHEST_PYTHON_2_VERSION = [2, 1, 0]
 
     def __init__(
         self,
         training_steps=None,
         evaluation_steps=None,
         checkpoint_path=None,
-        py_version="py2",
+        py_version=None,
         framework_version=None,
         model_dir=None,
         requirements_file="",
@@ -212,32 +223,47 @@ class TensorFlow(Framework):
         """Initialize a ``TensorFlow`` estimator.
 
         Args:
-            training_steps (int): Perform this many steps of training. `None`, the default means train forever.
-            evaluation_steps (int): Perform this many steps of evaluation. `None`, the default means that evaluation
-                runs until input from eval_input_fn is exhausted (or another exception is raised).
-            checkpoint_path (str): Identifies S3 location where checkpoint data during model training can be
-                saved (default: None). For distributed model training, this parameter is required.
-            py_version (str): Python version you want to use for executing your model training code (default: 'py2').
-            framework_version (str): TensorFlow version you want to use for executing your model training code.
-                List of supported versions https://github.com/aws/sagemaker-python-sdk#tensorflow-sagemaker-estimators.
+            training_steps (int): Perform this many steps of training. `None`, the default means
+                train forever.
+            evaluation_steps (int): Perform this many steps of evaluation. `None`, the default
+                means that evaluation runs until input from eval_input_fn is exhausted (or another
+                exception is raised).
+            checkpoint_path (str): Identifies S3 location where checkpoint data during model
+                training can be saved (default: None). For distributed model training, this
+                parameter is required.
+            py_version (str): Python version you want to use for executing your model training
+                code (default: 'py2').
+            framework_version (str): TensorFlow version you want to use for executing your model
+                training code. List of supported versions
+                https://github.com/aws/sagemaker-python-sdk#tensorflow-sagemaker-estimators.
                 If not specified, this will default to 1.11.
-            model_dir (str): S3 location where the checkpoint data and models can be exported to during training
-                (default: None). If not specified a default S3 URI will be generated. It will be passed in the
-                training script as one of the command line arguments.
-            requirements_file (str): Path to a ``requirements.txt`` file (default: ''). The path should be within and
-                relative to ``source_dir``. Details on the format can be found in the
-                `Pip User Guide <https://pip.pypa.io/en/stable/reference/pip_install/#requirements-file-format>`_.
-            image_name (str): If specified, the estimator will use this image for training and hosting, instead of
-                selecting the appropriate SageMaker official image based on framework_version and py_version. It can
-                be an ECR url or dockerhub image and tag.
+            model_dir (str): S3 location where the checkpoint data and models can be exported to
+                during training (default: None). It will be passed in the training script as one of
+                the command line arguments. If not specified, one is provided based on
+                your training configuration:
+
+                * *distributed training with MPI* - ``/opt/ml/model``
+                * *single-machine training or distributed training without MPI* - \
+                    ``s3://{output_path}/model``
+                * *Local Mode with local sources (file:// instead of s3://)* - \
+                    ``/opt/ml/shared/model``
+
+            requirements_file (str): Path to a ``requirements.txt`` file (default: ''). The path
+                should be within and relative to ``source_dir``. Details on the format can be
+                found in the Pip User Guide:
+                <https://pip.pypa.io/en/stable/reference/pip_install/#requirements-file-format>
+            image_name (str): If specified, the estimator will use this image for training and
+                hosting, instead of selecting the appropriate SageMaker official image based on
+                framework_version and py_version. It can be an ECR url or dockerhub image and tag.
 
                 Examples:
                     123.dkr.ecr.us-west-2.amazonaws.com/my-custom-image:1.0
                     custom-image:latest.
-            script_mode (bool): If set to True will the estimator will use the Script Mode containers (default: False).
-                This will be ignored if py_version is set to 'py3'.
+            script_mode (bool): If set to True will the estimator will use the Script Mode
+                containers (default: False). This will be ignored if py_version is set to 'py3'.
             distributions (dict): A dictionary with information on how to run distributed training
-                (default: None). Currently we support distributed training with parameter servers and MPI.
+                (default: None). Currently we support distributed training with parameter servers
+                and MPI.
                 To enable parameter server use the following setup:
 
                 .. code:: python
@@ -261,17 +287,40 @@ class TensorFlow(Framework):
                     }
 
             **kwargs: Additional kwargs passed to the Framework constructor.
+
+        .. tip::
+
+            You can find additional parameters for initializing this class at
+            :class:`~sagemaker.estimator.Framework` and
+            :class:`~sagemaker.estimator.EstimatorBase`.
         """
         if framework_version is None:
-            logger.warning(fw.empty_framework_version_warning(TF_VERSION, self.LATEST_VERSION))
-        self.framework_version = framework_version or TF_VERSION
+            logger.warning(
+                fw.empty_framework_version_warning(defaults.TF_VERSION, self.LATEST_VERSION)
+            )
+        self.framework_version = framework_version or defaults.TF_VERSION
+
+        if not py_version:
+            py_version = "py3" if self._only_python_3_supported() else "py2"
+        if py_version == "py2":
+            logger.warning(
+                fw.python_deprecation_warning(self.__framework_name__, defaults.LATEST_PY2_VERSION)
+            )
+
+        if distributions is not None:
+            train_instance_type = kwargs.get("train_instance_type")
+            fw.warn_if_parameter_server_with_multi_gpu(
+                training_instance_type=train_instance_type, distributions=distributions
+            )
+
+        if "enable_sagemaker_metrics" not in kwargs:
+            # enable sagemaker metrics for TF v1.15 or greater:
+            if fw.is_version_equal_or_higher([1, 15], self.framework_version):
+                kwargs["enable_sagemaker_metrics"] = True
 
         super(TensorFlow, self).__init__(image_name=image_name, **kwargs)
         self.checkpoint_path = checkpoint_path
 
-        if py_version == "py2":
-            logger.warning("tensorflow py2 container will be deprecated soon.")
-
         self.py_version = py_version
         self.training_steps = training_steps
         self.evaluation_steps = evaluation_steps
@@ -282,7 +331,7 @@ class TensorFlow(Framework):
         self._validate_args(
             py_version=py_version,
             script_mode=script_mode,
-            framework_version=framework_version,
+            framework_version=self.framework_version,
             training_steps=training_steps,
             evaluation_steps=evaluation_steps,
             requirements_file=requirements_file,
@@ -301,6 +350,7 @@ class TensorFlow(Framework):
         requirements_file,
         checkpoint_path,
     ):
+        """Placeholder docstring"""
 
         if py_version == "py3" or script_mode:
 
@@ -323,6 +373,13 @@ class TensorFlow(Framework):
                     )
                 )
 
+        if py_version == "py2" and self._only_python_3_supported():
+            msg = (
+                "Python 2 containers are only available with {} and lower versions. "
+                "Please use a Python 3 container.".format(defaults.LATEST_PY2_VERSION)
+            )
+            raise AttributeError(msg)
+
         if (not self._script_mode_enabled()) and self._only_script_mode_supported():
             logger.warning(
                 "Legacy mode is deprecated in versions 1.13 and higher. Using script mode instead."
@@ -330,11 +387,17 @@ class TensorFlow(Framework):
             self.script_mode = True
 
     def _only_script_mode_supported(self):
+        """Placeholder docstring"""
         return [
             int(s) for s in self.framework_version.split(".")
         ] >= self._LOWEST_SCRIPT_MODE_ONLY_VERSION
 
+    def _only_python_3_supported(self):
+        """Placeholder docstring"""
+        return [int(s) for s in self.framework_version.split(".")] > self._HIGHEST_PYTHON_2_VERSION
+
     def _validate_requirements_file(self, requirements_file):
+        """Placeholder docstring"""
         if not requirements_file:
             return
 
@@ -354,7 +417,15 @@ class TensorFlow(Framework):
         if not os.path.exists(os.path.join(self.source_dir, requirements_file)):
             raise ValueError("Requirements file {} does not exist.".format(requirements_file))
 
-    def fit(self, inputs=None, wait=True, logs=True, job_name=None, run_tensorboard_locally=False):
+    def fit(
+        self,
+        inputs=None,
+        wait=True,
+        logs=True,
+        job_name=None,
+        experiment_config=None,
+        run_tensorboard_locally=False,
+    ):
         """Train a model using the input training dataset.
 
         See :func:`~sagemaker.estimator.EstimatorBase.fit` for more details.
@@ -364,31 +435,33 @@ class TensorFlow(Framework):
                 This can be one of three types:
 
                 * (str) - the S3 location where training data is saved.
-                * (dict[str, str] or dict[str, sagemaker.session.s3_input]) - If using multiple channels for
-                    training data, you can specify a dict mapping channel names
+                * (dict[str, str] or dict[str, sagemaker.session.s3_input]) - If using multiple
+                    channels for training data, you can specify a dict mapping channel names
                     to strings or :func:`~sagemaker.session.s3_input` objects.
-                * (sagemaker.session.s3_input) - channel configuration for S3 data sources that can provide
-                    additional information as well as the path to the training dataset.
+                * (sagemaker.session.s3_input) - channel configuration for S3 data sources that
+                    can provide additional information as well as the path to the training dataset.
                     See :func:`sagemaker.session.s3_input` for full details.
 
             wait (bool): Whether the call should wait until the job completes (default: True).
             logs (bool): Whether to show the logs produced by the job.
                 Only meaningful when wait is True (default: True).
-            job_name (str): Training job name. If not specified, the estimator generates a default job name,
-                based on the training image name and current timestamp.
-            run_tensorboard_locally (bool): Whether to execute TensorBoard in a different process with
-                downloaded checkpoint information (default: False). This is an experimental feature, and requires
-                TensorBoard and AWS CLI to be installed. It terminates TensorBoard when execution ends.
+            job_name (str): Training job name. If not specified, the estimator generates a default
+                job name, based on the training image name and current timestamp.
+            experiment_config (dict[str, str]): Experiment management configuration.
+            run_tensorboard_locally (bool): Whether to execute TensorBoard in a different process
+                with downloaded checkpoint information (default: False). This is an experimental
+                feature, and requires TensorBoard and AWS CLI to be installed. It terminates
+                TensorBoard when execution ends.
         """
 
         def fit_super():
-            super(TensorFlow, self).fit(inputs, wait, logs, job_name)
+            super(TensorFlow, self).fit(inputs, wait, logs, job_name, experiment_config)
 
         if run_tensorboard_locally and wait is False:
             raise ValueError("Tensorboard is not supported with async fit")
 
         if self._script_mode_enabled() and run_tensorboard_locally:
-            logger.warning(_SCRIPT_MODE_TENSORBOARD_WARNING.format(self.model_dir))
+            logger.warning(_SCRIPT_MODE_TENSORBOARD_WARNING, self.model_dir)
             fit_super()
         elif run_tensorboard_locally:
             tensorboard = Tensorboard(self)
@@ -420,7 +493,8 @@ class TensorFlow(Framework):
             job_details, model_channel_name
         )
 
-        # Move some of the tensorflow specific init params from hyperparameters into the main init params.
+        # Move some of the tensorflow specific init params from hyperparameters into the main init
+        # params.
         for argument in ("checkpoint_path", "training_steps", "evaluation_steps", "model_dir"):
             value = init_params["hyperparameters"].pop(argument, None)
             if value is not None:
@@ -439,10 +513,10 @@ class TensorFlow(Framework):
 
         init_params["py_version"] = py_version
 
-        # We switched image tagging scheme from regular image version (e.g. '1.0') to more expressive
-        # containing framework version, device type and python version (e.g. '1.5-gpu-py2').
-        # For backward compatibility map deprecated image tag '1.0' to a '1.4' framework version
-        # otherwise extract framework version from the tag itself.
+        # We switched image tagging scheme from regular image version (e.g. '1.0') to more
+        # expressive containing framework version, device type and python version
+        # (e.g. '1.5-gpu-py2'). For backward compatibility map deprecated image tag '1.0' to a
+        # '1.4' framework version otherwise extract framework version from the tag itself.
         init_params["framework_version"] = (
             "1.4" if tag == "1.0" else fw.framework_version_from_tag(tag)
         )
@@ -463,59 +537,132 @@ class TensorFlow(Framework):
         role=None,
         vpc_config_override=VPC_CONFIG_DEFAULT,
         endpoint_type=None,
+        entry_point=None,
+        source_dir=None,
+        dependencies=None,
+        **kwargs
     ):
-        """Create a SageMaker ``TensorFlowModel`` object that can be deployed to an ``Endpoint``.
+        """Create a ``Model`` object that can be used for creating SageMaker model entities,
+        deploying to a SageMaker endpoint, or starting SageMaker Batch Transform jobs.
 
         Args:
-            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also used during
-                transform jobs. If not specified, the role from the Estimator will be used.
-            model_server_workers (int): Optional. The number of worker processes used by the inference server.
-                If None, server will use one worker per vCPU.
-            vpc_config_override (dict[str, list[str]]): Optional override for VpcConfig set on the model.
+            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also
+                used during transform jobs. If not specified, the role from the Estimator will be
+                used.
+            model_server_workers (int): Optional. The number of worker processes used by the
+                inference server. If None, server will use one worker per vCPU.
+            vpc_config_override (dict[str, list[str]]): Optional override for VpcConfig set on the
+                model.
                 Default: use subnets and security groups from this Estimator.
                 * 'Subnets' (list[str]): List of subnet ids.
                 * 'SecurityGroupIds' (list[str]): List of security group ids.
-            endpoint_type: Optional. Selects the software stack used by the inference server.
+            endpoint_type (str): Optional. Selects the software stack used by the inference server.
                 If  not specified, the model will be configured to use the default
                 SageMaker model server. If 'tensorflow-serving', the model will be configured to
                 use the SageMaker Tensorflow Serving container.
+            entry_point (str): Path (absolute or relative) to the local Python source file which
+                should be executed as the entry point to training. If not specified and
+                ``endpoint_type`` is 'tensorflow-serving', no entry point is used. If
+                ``endpoint_type`` is also ``None``, then the training entry point is used.
+            source_dir (str): Path (absolute or relative) to a directory with any other serving
+                source code dependencies aside from the entry point file. If not specified and
+                ``endpoint_type`` is 'tensorflow-serving', no source_dir is used. If
+                ``endpoint_type`` is also ``None``, then the model source directory from training
+                is used.
+            dependencies (list[str]): A list of paths to directories (absolute or relative) with
+                any additional libraries that will be exported to the container.
+                If not specified and ``endpoint_type`` is 'tensorflow-serving', ``dependencies`` is
+                set to ``None``.
+                If ``endpoint_type`` is also ``None``, then the dependencies from training are used.
+            **kwargs: Additional kwargs passed to :class:`~sagemaker.tensorflow.serving.Model`
+                and :class:`~sagemaker.tensorflow.model.TensorFlowModel` constructors.
 
         Returns:
-            sagemaker.tensorflow.model.TensorFlowModel: A SageMaker ``TensorFlowModel`` object.
-                See :func:`~sagemaker.tensorflow.model.TensorFlowModel` for full details.
+            sagemaker.tensorflow.model.TensorFlowModel or sagemaker.tensorflow.serving.Model: A
+                ``Model`` object. See :class:`~sagemaker.tensorflow.serving.Model` or
+                :class:`~sagemaker.tensorflow.model.TensorFlowModel` for full details.
         """
-
         role = role or self.role
+
         if endpoint_type == "tensorflow-serving" or self._script_mode_enabled():
-            return self._create_tfs_model(role=role, vpc_config_override=vpc_config_override)
+            return self._create_tfs_model(
+                role=role,
+                vpc_config_override=vpc_config_override,
+                entry_point=entry_point,
+                source_dir=source_dir,
+                dependencies=dependencies,
+                **kwargs
+            )
 
         return self._create_default_model(
             model_server_workers=model_server_workers,
             role=role,
             vpc_config_override=vpc_config_override,
+            entry_point=entry_point,
+            source_dir=source_dir,
+            dependencies=dependencies,
+            **kwargs
         )
 
-    def _create_tfs_model(self, role=None, vpc_config_override=VPC_CONFIG_DEFAULT):
+    def _create_tfs_model(
+        self,
+        role=None,
+        vpc_config_override=VPC_CONFIG_DEFAULT,
+        entry_point=None,
+        source_dir=None,
+        dependencies=None,
+        **kwargs
+    ):
+        """Placeholder docstring"""
+        # remove image kwarg
+        if "image" in kwargs:
+            image = kwargs["image"]
+            kwargs = {k: v for k, v in kwargs.items() if k != "image"}
+        else:
+            image = None
+
         return Model(
             model_data=self.model_data,
             role=role,
-            image=self.image_name,
+            image=(image or self.image_name),
             name=self._current_job_name,
             container_log_level=self.container_log_level,
             framework_version=utils.get_short_version(self.framework_version),
             sagemaker_session=self.sagemaker_session,
             vpc_config=self.get_vpc_config(vpc_config_override),
+            entry_point=entry_point,
+            source_dir=source_dir,
+            dependencies=dependencies,
+            enable_network_isolation=self.enable_network_isolation(),
+            **kwargs
         )
 
-    def _create_default_model(self, model_server_workers, role, vpc_config_override):
+    def _create_default_model(
+        self,
+        model_server_workers,
+        role,
+        vpc_config_override,
+        entry_point=None,
+        source_dir=None,
+        dependencies=None,
+        **kwargs
+    ):
+        """Placeholder docstring"""
+        # remove image kwarg
+        if "image" in kwargs:
+            image = kwargs["image"]
+            kwargs = {k: v for k, v in kwargs.items() if k != "image"}
+        else:
+            image = None
+
         return TensorFlowModel(
             self.model_data,
             role,
-            self.entry_point,
-            source_dir=self._model_source_dir(),
+            entry_point or self.entry_point,
+            source_dir=source_dir or self._model_source_dir(),
             enable_cloudwatch_metrics=self.enable_cloudwatch_metrics,
             env={"SAGEMAKER_REQUIREMENTS": self.requirements_file},
-            image=self.image_name,
+            image=(image or self.image_name),
             name=self._current_job_name,
             container_log_level=self.container_log_level,
             code_location=self.code_location,
@@ -524,7 +671,9 @@ class TensorFlow(Framework):
             model_server_workers=model_server_workers,
             sagemaker_session=self.sagemaker_session,
             vpc_config=self.get_vpc_config(vpc_config_override),
-            dependencies=self.dependencies,
+            dependencies=dependencies or self.dependencies,
+            enable_network_isolation=self.enable_network_isolation(),
+            **kwargs
         )
 
     def hyperparameters(self):
@@ -545,9 +694,12 @@ class TensorFlow(Framework):
                 mpi_dict = self.distributions["mpi"]
                 mpi_enabled = mpi_dict.get("enabled", False)
                 additional_hyperparameters[self.LAUNCH_MPI_ENV_NAME] = mpi_enabled
-                additional_hyperparameters[self.MPI_NUM_PROCESSES_PER_HOST] = mpi_dict.get(
-                    "processes_per_host", 1
-                )
+
+                if mpi_dict.get("processes_per_host"):
+                    additional_hyperparameters[self.MPI_NUM_PROCESSES_PER_HOST] = mpi_dict.get(
+                        "processes_per_host"
+                    )
+
                 additional_hyperparameters[self.MPI_CUSTOM_MPI_OPTIONS] = mpi_dict.get(
                     "custom_mpi_options", ""
                 )
@@ -566,20 +718,46 @@ class TensorFlow(Framework):
         return hyperparameters
 
     def _default_s3_path(self, directory, mpi=False):
+        """Placeholder docstring"""
         local_code = utils.get_config_value("local.local_code", self.sagemaker_session.config)
         if self.sagemaker_session.local_mode and local_code:
             return "/opt/ml/shared/{}".format(directory)
-        elif mpi:
+        if mpi:
             return "/opt/ml/model"
-        elif self._current_job_name:
+        if self._current_job_name:
             return os.path.join(self.output_path, self._current_job_name, directory)
-        else:
-            return None
+        return None
 
     def _script_mode_enabled(self):
+        """Placeholder docstring"""
         return self.py_version == "py3" or self.script_mode
 
+    def _validate_and_set_debugger_configs(self):
+        """
+        Disable Debugger Hook Config for ParameterServer (PS) as it is not
+        supported in smdebug.
+
+        Else, set default HookConfig
+        """
+        ps_enabled = "parameter_server" in self.distributions and self.distributions[
+            "parameter_server"
+        ].get("enabled", False)
+        if ps_enabled:
+            if self.debugger_hook_config is not None or self.debugger_rule_configs is not None:
+                logger.info(
+                    "Amazon SageMaker Debugger does not currently support "
+                    "Parameter Server distribution"
+                )
+            self.debugger_hook_config = None
+            self.debugger_rule_configs = None
+        elif self.debugger_hook_config is None and fw._region_supports_debugger(
+            self.sagemaker_session.boto_session.region_name
+        ):
+            # Set defaults for debugging.
+            self.debugger_hook_config = DebuggerHookConfig(s3_output_path=self.output_path)
+
     def train_image(self):
+        """Placeholder docstring"""
         if self.image_name:
             return self.image_name
 
@@ -611,30 +789,39 @@ class TensorFlow(Framework):
         model_server_workers=None,
         volume_kms_key=None,
         endpoint_type=None,
+        entry_point=None,
+        vpc_config_override=VPC_CONFIG_DEFAULT,
     ):
-        """Return a ``Transformer`` that uses a SageMaker Model based on the training job. It reuses the
-        SageMaker Session and base job name used by the Estimator.
+        """Return a ``Transformer`` that uses a SageMaker Model based on the training job. It
+        reuses the SageMaker Session and base job name used by the Estimator.
 
         Args:
             instance_count (int): Number of EC2 instances to use.
             instance_type (str): Type of EC2 instance to use, for example, 'ml.c4.xlarge'.
-            strategy (str): The strategy used to decide how to batch records in a single request (default: None).
-                Valid values: 'MULTI_RECORD' and 'SINGLE_RECORD'.
-            assemble_with (str): How the output is assembled (default: None). Valid values: 'Line' or 'None'.
-            output_path (str): S3 location for saving the transform result. If not specified, results are stored to
-                a default bucket.
-            output_kms_key (str): Optional. KMS key ID for encrypting the transform output (default: None).
-            accept (str): The content type accepted by the endpoint deployed during the transform job.
-            env (dict): Environment variables to be set for use during the transform job (default: None).
+            strategy (str): The strategy used to decide how to batch records in a single request
+                (default: None). Valid values: 'MultiRecord' and 'SingleRecord'.
+            assemble_with (str): How the output is assembled (default: None). Valid values: 'Line'
+                or 'None'.
+            output_path (str): S3 location for saving the transform result. If not specified,
+                results are stored to a default bucket.
+            output_kms_key (str): Optional. KMS key ID for encrypting the transform output
+                (default: None).
+            accept (str): The accept header passed by the client to
+                the inference endpoint. If it is supported by the endpoint,
+                it will be the format of the batch transform output.
+            env (dict): Environment variables to be set for use during the transform job
+                (default: None).
             max_concurrent_transforms (int): The maximum number of HTTP requests to be made to
                 each individual transform container at one time.
-            max_payload (int): Maximum size of the payload in a single HTTP request to the container in MB.
-            tags (list[dict]): List of tags for labeling a transform job. If none specified, then the tags used for
-                the training job are used for the transform job.
-            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also used during
-                transform jobs. If not specified, the role from the Estimator will be used.
-            model_server_workers (int): Optional. The number of worker processes used by the inference server.
-                If None, server will use one worker per vCPU.
+            max_payload (int): Maximum size of the payload in a single HTTP request to the
+                container in MB.
+            tags (list[dict]): List of tags for labeling a transform job. If none specified, then
+                the tags used for the training job are used for the transform job.
+            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also
+                used during transform jobs. If not specified, the role from the Estimator will be
+                used.
+            model_server_workers (int): Optional. The number of worker processes used by the
+                inference server. If None, server will use one worker per vCPU.
             volume_kms_key (str): Optional. KMS key ID for encrypting the volume attached to the ML
                 compute instance (default: None).
             endpoint_type (str): Optional. Selects the software stack used by the inference server.
@@ -642,14 +829,47 @@ class TensorFlow(Framework):
                 SageMaker model server.
                 If 'tensorflow-serving', the model will be configured to
                 use the SageMaker Tensorflow Serving container.
+            entry_point (str): Path (absolute or relative) to the local Python source file which
+                should be executed as the entry point to training. If not specified and
+                ``endpoint_type`` is 'tensorflow-serving', no entry point is used. If
+                ``endpoint_type`` is also ``None``, then the training entry point is used.
+            vpc_config_override (dict[str, list[str]]): Optional override for
+                the VpcConfig set on the model.
+                Default: use subnets and security groups from this Estimator.
+                * 'Subnets' (list[str]): List of subnet ids.
+                * 'SecurityGroupIds' (list[str]): List of security group ids.
         """
-
         role = role or self.role
+
+        if self.latest_training_job is None:
+            logging.warning(
+                "No finished training job found associated with this estimator. Please make sure "
+                "this estimator is only used for building workflow config"
+            )
+            return Transformer(
+                self._current_job_name,
+                instance_count,
+                instance_type,
+                strategy=strategy,
+                assemble_with=assemble_with,
+                output_path=output_path,
+                output_kms_key=output_kms_key,
+                accept=accept,
+                max_concurrent_transforms=max_concurrent_transforms,
+                max_payload=max_payload,
+                env=env or {},
+                tags=tags,
+                base_transform_job_name=self.base_job_name,
+                volume_kms_key=volume_kms_key,
+                sagemaker_session=self.sagemaker_session,
+            )
+
         model = self.create_model(
             model_server_workers=model_server_workers,
             role=role,
-            vpc_config_override=VPC_CONFIG_DEFAULT,
+            vpc_config_override=vpc_config_override,
             endpoint_type=endpoint_type,
+            entry_point=entry_point,
         )
         return model.transformer(
             instance_count,
