@@ -1,4 +1,4 @@
-# Copyright 2017-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,10 +10,12 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Placeholder docstring"""
 from __future__ import absolute_import
 
 import contextlib
 import errno
+import logging
 import os
 import random
 import re
@@ -22,14 +24,22 @@ import sys
 import tarfile
 import tempfile
 import time
-
 from datetime import datetime
 from functools import wraps
+
+import botocore
+import six
 from six.moves.urllib import parse
 
-import six
 
 ECR_URI_PATTERN = r"^(\d+)(\.)dkr(\.)ecr(\.)(.+)(\.)(amazonaws.com|c2s.ic.gov)(/)(.*:.*)$"
+MAX_BUCKET_PATHS_COUNT = 5
+S3_PREFIX = "s3://"
+HTTP_PREFIX = "http://"
+HTTPS_PREFIX = "https://"
+DEFAULT_SLEEP_TIME_SECONDS = 10
+
+logger = logging.getLogger(__name__)
 
 
 # Use the base name of the image as the job name if the user doesn't give us one
@@ -40,7 +50,8 @@ def name_from_image(image):
         image (str): Image name.
 
     Returns:
-        str: Training job name using the algorithm from the image name and a timestamp.
+        str: Training job name using the algorithm from the image name and a
+        timestamp.
     """
     return name_from_base(base_name_from_image(image))
 
@@ -48,8 +59,9 @@ def name_from_image(image):
 def name_from_base(base, max_length=63, short=False):
     """Append a timestamp to the provided string.
 
-    This function assures that the total length of the resulting string is not
-    longer than the specified max length, trimming the input parameter if necessary.
+    This function assures that the total length of the resulting string is
+    not longer than the specified max length, trimming the input parameter if
+    necessary.
 
     Args:
         base (str): String used as prefix to generate the unique name.
@@ -65,6 +77,11 @@ def name_from_base(base, max_length=63, short=False):
 
 
 def unique_name_from_base(base, max_length=63):
+    """
+    Args:
+        base:
+        max_length:
+    """
     unique = "%04x" % random.randrange(16 ** 4)  # 4-digit hex
     ts = str(int(time.time()))
     available_length = max_length - 2 - len(ts) - len(unique)
@@ -73,7 +90,8 @@ def unique_name_from_base(base, max_length=63):
 
 
 def base_name_from_image(image):
-    """Extract the base name of the image to use as the 'algorithm name' for the job.
+    """Extract the base name of the image to use as the 'algorithm name' for the
+    job.
 
     Args:
         image (str): Image name.
@@ -99,7 +117,11 @@ def sagemaker_short_timestamp():
 
 
 def debug(func):
-    """Print the function name and arguments for debugging."""
+    """Print the function name and arguments for debugging.
+
+    Args:
+        func:
+    """
 
     @wraps(func)
     def wrapper(*args, **kwargs):
@@ -110,6 +132,11 @@ def debug(func):
 
 
 def get_config_value(key_path, config):
+    """
+    Args:
+        key_path:
+        config:
+    """
     if config is None:
         return None
 
@@ -138,15 +165,16 @@ def get_short_version(framework_version):
 def to_str(value):
     """Convert the input to a string, unless it is a unicode string in Python 2.
 
-    Unicode strings are supported as native strings in Python 3, but ``str()`` cannot be
-    invoked on unicode strings in Python 2, so we need to check for that case when
-    converting user-specified values to strings.
+    Unicode strings are supported as native strings in Python 3, but
+    ``str()`` cannot be invoked on unicode strings in Python 2, so we need to
+    check for that case when converting user-specified values to strings.
 
     Args:
         value: The value to convert to a string.
 
     Returns:
-        str or unicode: The string representation of the value or the unicode string itself.
+        str or unicode: The string representation of the value or the unicode
+        string itself.
     """
     if sys.version_info.major < 3 and isinstance(value, six.string_types):
         return value
@@ -154,8 +182,11 @@ def to_str(value):
 
 
 def extract_name_from_job_arn(arn):
-    """Returns the name used in the API given a full ARN for a training job
-    or hyperparameter tuning job.
+    """Returns the name used in the API given a full ARN for a training job or
+    hyperparameter tuning job.
+
+    Args:
+        arn:
     """
     slash_pos = arn.find("/")
     if slash_pos == -1:
@@ -167,12 +198,12 @@ def secondary_training_status_changed(current_job_description, prev_job_descript
     """Returns true if training job's secondary status message has changed.
 
     Args:
-        current_job_desc: Current job description, returned from DescribeTrainingJob call.
-        prev_job_desc: Previous job description, returned from DescribeTrainingJob call.
+        current_job_description: Current job description, returned from DescribeTrainingJob call.
+        prev_job_description: Previous job description, returned from DescribeTrainingJob call.
 
     Returns:
-        boolean: Whether the secondary status message of a training job changed or not.
-
+        boolean: Whether the secondary status message of a training job changed
+        or not.
     """
     current_secondary_status_transitions = current_job_description.get("SecondaryStatusTransitions")
     if (
@@ -200,7 +231,8 @@ def secondary_training_status_changed(current_job_description, prev_job_descript
 
 
 def secondary_training_status_message(job_description, prev_description):
-    """Returns a string contains last modified time and the secondary training job status message.
+    """Returns a string contains last modified time and the secondary training
+    job status message.
 
     Args:
         job_description: Returned response from DescribeTrainingJob call
@@ -208,7 +240,6 @@ def secondary_training_status_message(job_description, prev_description):
 
     Returns:
         str: Job status string to be printed.
-
     """
 
     if (
@@ -248,31 +279,95 @@ def secondary_training_status_message(job_description, prev_description):
     return "\n".join(status_strs)
 
 
+def generate_tensorboard_url(domain, bucket_paths):
+    """Generate Tensorboard URL for given list of s3 buckets
+
+    Args:
+        domain: JupyterLab app domain
+        bucket_paths: List of S3 bucket paths in format `bucket/path`
+                      or a single string in the same format
+
+    Returns:
+        str: Tensorboard URL
+
+    Raises:
+        AttributeError if invalid inputs are passed
+    """
+
+    def trim_prefix(s, prefix):
+        if s.startswith(prefix):
+            return s[len(prefix) :]
+        return s
+
+    def encode_s3_url(s3_url):
+        if not s3_url:
+            raise AttributeError("bucket_paths element should not be empty")
+        s3_url = trim_prefix(s3_url, S3_PREFIX)
+        return parse.quote_plus("{}{}".format(S3_PREFIX, s3_url))
+
+    if not isinstance(domain, six.string_types):
+        raise AttributeError("domain parameter should be string")
+
+    if len(domain) == 0:
+        raise AttributeError("domain parameter should not be empty")
+
+    if isinstance(bucket_paths, six.string_types):
+        bucket_paths = [bucket_paths]
+    elif not isinstance(bucket_paths, list):
+        raise AttributeError("bucket paths should be a list or a string")
+
+    if len(bucket_paths) == 0:
+        raise AttributeError("bucket_paths parameter should not be empty list")
+
+    domain = trim_prefix(domain, HTTPS_PREFIX)
+    domain = trim_prefix(domain, HTTP_PREFIX)
+
+    s3_urls = map(encode_s3_url, bucket_paths)
+    query = ",".join(s3_urls)
+
+    return "https://{}/tensorboard/default?s3urls={}".format(domain, query)
+
+
 def download_folder(bucket_name, prefix, target, sagemaker_session):
     """Download a folder from S3 to a local path
 
     Args:
         bucket_name (str): S3 bucket name
-        prefix (str): S3 prefix within the bucket that will be downloaded. Can be a single file.
+        prefix (str): S3 prefix within the bucket that will be downloaded. Can
+            be a single file.
         target (str): destination path where the downloaded items will be placed
-        sagemaker_session (:class:`sagemaker.session.Session`): a sagemaker session to interact with S3.
+        sagemaker_session (sagemaker.session.Session): a sagemaker session to
+            interact with S3.
     """
     boto_session = sagemaker_session.boto_session
-
-    s3 = boto_session.resource("s3")
-    bucket = s3.Bucket(bucket_name)
+    s3 = boto_session.resource("s3", region_name=boto_session.region_name)
 
     prefix = prefix.lstrip("/")
 
-    # there is a chance that the prefix points to a file and not a 'directory' if that is the case
-    # we should just download it.
-    objects = list(bucket.objects.filter(Prefix=prefix))
-
-    if len(objects) > 0 and objects[0].key == prefix and prefix[-1] != "/":
+    # Try to download the prefix as an object first, in case it is a file and not a 'directory'.
+    # Do this first, in case the object has broader permissions than the bucket.
+    try:
         s3.Object(bucket_name, prefix).download_file(os.path.join(target, os.path.basename(prefix)))
         return
+    except botocore.exceptions.ClientError as e:
+        if e.response["Error"]["Code"] == "404" and e.response["Error"]["Message"] == "Not Found":
+            # S3 also throws this error if the object is a folder,
+            # so assume that is the case here, and then raise for an actual 404 later.
+            _download_files_under_prefix(bucket_name, prefix, target, s3)
+        else:
+            raise
 
-    # the prefix points to an s3 'directory' download the whole thing
+
+def _download_files_under_prefix(bucket_name, prefix, target, s3):
+    """Download all S3 files which match the given prefix
+
+    Args:
+        bucket_name (str): S3 bucket name
+        prefix (str): S3 prefix within the bucket that will be downloaded
+        target (str): destination path where the downloaded items will be placed
+        s3 (boto3.resources.base.ServiceResource): S3 resource
+    """
+    bucket = s3.Bucket(bucket_name)
     for obj_sum in bucket.objects.filter(Prefix=prefix):
         # if obj_sum is a folder object skip it.
         if obj_sum.key != "" and obj_sum.key[-1] == "/":
@@ -293,10 +388,13 @@ def download_folder(bucket_name, prefix, target, sagemaker_session):
 
 def create_tar_file(source_files, target=None):
     """Create a tar file containing all the source_files
+
     Args:
-        source_files (List[str]): List of file paths that will be contained in the tar file
+        source_files: (List[str]): List of file paths that will be contained in the tar file
+        target:
+
     Returns:
-         (str): path to created tar file
+        (str): path to created tar file
     """
     if target:
         filename = target
@@ -312,17 +410,17 @@ def create_tar_file(source_files, target=None):
 
 @contextlib.contextmanager
 def _tmpdir(suffix="", prefix="tmp"):
-    """Create a temporary directory with a context manager. The file is deleted when the context exits.
+    """Create a temporary directory with a context manager. The file is deleted
+    when the context exits.
 
     The prefix, suffix, and dir arguments are the same as for mkstemp().
 
     Args:
-        suffix (str):  If suffix is specified, the file name will end with that suffix, otherwise there will be no
-                        suffix.
-        prefix (str):  If prefix is specified, the file name will begin with that prefix; otherwise,
-                        a default prefix is used.
-        dir (str):  If dir is specified, the file will be created in that directory; otherwise, a default directory is
-                        used.
+        suffix (str): If suffix is specified, the file name will end with that
+            suffix, otherwise there will be no suffix.
+        prefix (str): If prefix is specified, the file name will begin with that
+            prefix; otherwise, a default prefix is used.
+
     Returns:
         str: path to the directory
     """
@@ -338,36 +436,43 @@ def repack_model(
     model_uri,
     repacked_model_uri,
     sagemaker_session,
+    kms_key=None,
 ):
-    """Unpack model tarball and creates a new model tarball with the provided code script.
+    """Unpack model tarball and creates a new model tarball with the provided
+    code script.
 
-    This function does the following:
-    - uncompresses model tarball from S3 or local system into a temp folder
-    - replaces the inference code from the model with the new code provided
-    - compresses the new model tarball and saves it in S3 or local file system
+    This function does the following: - uncompresses model tarball from S3 or
+    local system into a temp folder - replaces the inference code from the model
+    with the new code provided - compresses the new model tarball and saves it
+    in S3 or local file system
 
     Args:
-        inference_script (str): path or basename of the inference script that will be packed into the model
-        source_directory (str): path including all the files that will be packed into the model
-        dependencies (list[str]): A list of paths to directories (absolute or relative) with
-                any additional libraries that will be exported to the container (default: []).
-                The library folders will be copied to SageMaker in the same folder where the entrypoint is copied.
-                Example:
+        inference_script (str): path or basename of the inference script that
+            will be packed into the model
+        source_directory (str): path including all the files that will be packed
+            into the model
+        dependencies (list[str]): A list of paths to directories (absolute or
+            relative) with any additional libraries that will be exported to the
+            container (default: []). The library folders will be copied to
+            SageMaker in the same folder where the entrypoint is copied.
+            Example
 
-                    The following call
-                    >>> Estimator(entry_point='train.py', dependencies=['my/libs/common', 'virtual-env'])
-                    results in the following inside the container:
+                The following call >>> Estimator(entry_point='train.py',
+                dependencies=['my/libs/common', 'virtual-env']) results in the
+                following inside the container:
 
-                    >>> $ ls
+                >>> $ ls
 
-                    >>> opt/ml/code
-                    >>>     |------ train.py
-                    >>>     |------ common
-                    >>>     |------ virtual-env
-
-        repacked_model_uri (str): path or file system location where the new model will be saved
+                >>> opt/ml/code
+                >>>     |------ train.py
+                >>>     |------ common
+                >>>     |------ virtual-env
         model_uri (str): S3 or file system location of the original model tar
-        sagemaker_session (:class:`sagemaker.session.Session`): a sagemaker session to interact with S3.
+        repacked_model_uri (str): path or file system location where the new
+            model will be saved
+        sagemaker_session (sagemaker.session.Session): a sagemaker session to
+            interact with S3.
+        kms_key (str): KMS key ARN for encrypting the repacked model file
 
     Returns:
         str: path to the new packed model
@@ -385,18 +490,28 @@ def repack_model(
         with tarfile.open(tmp_model_path, mode="w:gz") as t:
             t.add(model_dir, arcname=os.path.sep)
 
-        _save_model(repacked_model_uri, tmp_model_path, sagemaker_session)
+        _save_model(repacked_model_uri, tmp_model_path, sagemaker_session, kms_key=kms_key)
 
 
-def _save_model(repacked_model_uri, tmp_model_path, sagemaker_session):
+def _save_model(repacked_model_uri, tmp_model_path, sagemaker_session, kms_key):
+    """
+    Args:
+        repacked_model_uri:
+        tmp_model_path:
+        sagemaker_session:
+    """
     if repacked_model_uri.lower().startswith("s3://"):
         url = parse.urlparse(repacked_model_uri)
         bucket, key = url.netloc, url.path.lstrip("/")
         new_key = key.replace(os.path.basename(key), os.path.basename(repacked_model_uri))
 
-        sagemaker_session.boto_session.resource("s3").Object(bucket, new_key).upload_file(
-            tmp_model_path
-        )
+        if kms_key:
+            extra_args = {"ServerSideEncryption": "aws:kms", "SSEKMSKeyId": kms_key}
+        else:
+            extra_args = None
+        sagemaker_session.boto_session.resource(
+            "s3", region_name=sagemaker_session.boto_region_name
+        ).Object(bucket, new_key).upload_file(tmp_model_path, ExtraArgs=extra_args)
     else:
         shutil.move(tmp_model_path, repacked_model_uri.replace("file://", ""))
 
@@ -404,9 +519,20 @@ def _save_model(repacked_model_uri, tmp_model_path, sagemaker_session):
 def _create_or_update_code_dir(
     model_dir, inference_script, source_directory, dependencies, sagemaker_session, tmp
 ):
+    """
+    Args:
+        model_dir:
+        inference_script:
+        source_directory:
+        dependencies:
+        sagemaker_session:
+        tmp:
+    """
     code_dir = os.path.join(model_dir, "code")
     if os.path.exists(code_dir):
-        shutil.rmtree(code_dir, ignore_errors=True)
+        for filename in os.listdir(code_dir):
+            if filename.endswith(".py"):
+                os.remove(os.path.join(code_dir, filename))
     if source_directory and source_directory.lower().startswith("s3://"):
         local_code_path = os.path.join(tmp, "local_code.tar.gz")
         download_file_from_url(source_directory, local_code_path, sagemaker_session)
@@ -415,19 +541,31 @@ def _create_or_update_code_dir(
             t.extractall(path=code_dir)
 
     elif source_directory:
+        if os.path.exists(code_dir):
+            shutil.rmtree(code_dir)
         shutil.copytree(source_directory, code_dir)
     else:
-        os.mkdir(code_dir)
+        if not os.path.exists(code_dir):
+            os.mkdir(code_dir)
         shutil.copy2(inference_script, code_dir)
 
     for dependency in dependencies:
+        lib_dir = os.path.join(code_dir, "lib")
         if os.path.isdir(dependency):
-            shutil.copytree(dependency, code_dir)
+            shutil.copytree(dependency, os.path.join(lib_dir, os.path.basename(dependency)))
         else:
-            shutil.copy2(dependency, code_dir)
+            if not os.path.exists(lib_dir):
+                os.mkdir(lib_dir)
+            shutil.copy2(dependency, lib_dir)
 
 
 def _extract_model(model_uri, sagemaker_session, tmp):
+    """
+    Args:
+        model_uri:
+        sagemaker_session:
+        tmp:
+    """
     tmp_model_dir = os.path.join(tmp, "model")
     os.mkdir(tmp_model_dir)
     if model_uri.lower().startswith("s3://"):
@@ -441,6 +579,12 @@ def _extract_model(model_uri, sagemaker_session, tmp):
 
 
 def download_file_from_url(url, dst, sagemaker_session):
+    """
+    Args:
+        url:
+        dst:
+        sagemaker_session:
+    """
     url = parse.urlparse(url)
     bucket, key = url.netloc, url.path.lstrip("/")
 
@@ -454,12 +598,13 @@ def download_file(bucket_name, path, target, sagemaker_session):
         bucket_name (str): S3 bucket name
         path (str): file path within the bucket
         target (str): destination directory for the downloaded file.
-        sagemaker_session (:class:`sagemaker.session.Session`): a sagemaker session to interact with S3.
+        sagemaker_session (sagemaker.session.Session): a sagemaker session to
+            interact with S3.
     """
     path = path.lstrip("/")
     boto_session = sagemaker_session.boto_session
 
-    s3 = boto_session.resource("s3")
+    s3 = boto_session.resource("s3", region_name=sagemaker_session.boto_region_name)
     bucket = s3.Bucket(bucket_name)
     bucket.download_file(path, target)
 
@@ -474,15 +619,82 @@ def get_ecr_image_uri_prefix(account, region):
     Returns:
         (str): URI prefix of ECR image
     """
-    domain = "c2s.ic.gov" if region == "us-iso-east-1" else "amazonaws.com"
-    return "{}.dkr.ecr.{}.{}".format(account, region, domain)
+    endpoint_data = _botocore_resolver().construct_endpoint("ecr", region)
+    return "{}.dkr.{}".format(account, endpoint_data["hostname"])
+
+
+def sts_regional_endpoint(region):
+    """Get the AWS STS endpoint specific for the given region.
+
+    We need this function because the AWS SDK does not yet honor
+    the ``region_name`` parameter when creating an AWS STS client.
+
+    For the list of regional endpoints, see
+    https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html#id_credentials_region-endpoints.
+
+    Args:
+        region (str): AWS region name
+
+    Returns:
+        str: AWS STS regional endpoint
+    """
+    endpoint_data = _botocore_resolver().construct_endpoint("sts", region)
+    return "https://{}".format(endpoint_data["hostname"])
+
+
+def retries(max_retry_count, exception_message_prefix, seconds_to_sleep=DEFAULT_SLEEP_TIME_SECONDS):
+    """Retries until max retry count is reached.
+
+    Args:
+        max_retry_count (int): The retry count.
+        exception_message_prefix (str): The message to include in the exception on failure.
+        seconds_to_sleep (int): The number of seconds to sleep between executions.
+
+    """
+    for i in range(max_retry_count):
+        yield i
+        time.sleep(seconds_to_sleep)
+
+    raise Exception(
+        "'{}' has reached the maximum retry count of {}".format(
+            exception_message_prefix, max_retry_count
+        )
+    )
+
+
+def _botocore_resolver():
+    """Get the DNS suffix for the given region.
+
+    Args:
+        region (str): AWS region name
+
+    Returns:
+        str: the DNS suffix
+    """
+    loader = botocore.loaders.create_loader()
+    return botocore.regions.EndpointResolver(loader.load_data("endpoints"))
+
+
+def _aws_partition(region):
+    """
+    Given a region name (ex: "cn-north-1"), return the corresponding aws partition ("aws-cn").
+
+    Args:
+        region (str): The region name for which to return the corresponding partition.
+        Ex: "cn-north-1"
+
+    Returns:
+        str: partition corresponding to the region name passed in. Ex: "aws-cn"
+    """
+    endpoint_data = _botocore_resolver().construct_endpoint("sts", region)
+    return endpoint_data["partition"]
 
 
 class DeferredError(object):
-    """Stores an exception and raises it at a later time if this
-    object is accessed in any way.  Useful to allow soft-dependencies on imports,
-    so that the ImportError can be raised again later if code actually
-    relies on the missing library.
+    """Stores an exception and raises it at a later time if this object is
+    accessed in any way. Useful to allow soft-dependencies on imports, so that
+    the ImportError can be raised again later if code actually relies on the
+    missing library.
 
     Example::
 
@@ -494,11 +706,38 @@ class DeferredError(object):
     """
 
     def __init__(self, exception):
+        """
+        Args:
+            exception:
+        """
         self.exc = exception
 
     def __getattr__(self, name):
-        """Called by Python interpreter before using any method or property
-        on the object.  So this will short-circuit essentially any access to this
+        """Called by Python interpreter before using any method or property on
+        the object. So this will short-circuit essentially any access to this
         object.
+
+        Args:
+            name:
         """
         raise self.exc
+
+
+def _module_import_error(py_module, feature, extras):
+    """Return error message for module import errors, provide
+    installation details.
+
+    Args:
+        py_module (str): Module that failed to be imported
+        feature (str): Affected SageMaker feature
+        extras (str): Name of the `extras_require` to install the relevant dependencies
+
+    Returns:
+        str: Error message with installation instructions.
+    """
+    error_msg = (
+        "Failed to import {}. {} features will be impaired or broken. "
+        "Please run \"pip install 'sagemaker[{}]'\" "
+        "to install all required dependencies."
+    )
+    return error_msg.format(py_module, feature, extras)
